# _*_coding:utf-8_*_
# from django.conf.urls import url
from django.urls import re_path

from .apis.resource_file_list import UploadFileList
from .apis.resource_image_list import UploadImageList
from .apis.resource_upload_file import UploadFile
from .apis.resource_upload_image import UploadImage
from .apis.resource_upload_video import UploadVideo
from .apis.resource_video_list import UploadVideoList

# 应用名称
app_name = 'resource'

urlpatterns = [
    # 上传接口
    re_path(r'upload_file/?$', UploadFile.as_view(), name='st_upload_file'),  # 文件上传
    re_path(r'upload_image/?$', UploadImage.as_view(), name='st_upload_image'),  # 图片上传
    re_path(r'upload_video/?$', UploadVideo.as_view(), name='st_upload_image'),  # 图片上传
    # 列表接口
    re_path(r'image_list/?$', UploadImageList.as_view(), name='st_image_list'),  # 图片列表
    re_path(r'file_list/?$', UploadFileList.as_view(), name='st_file_list'),  # 文件列表
    re_path(r'video_list/?$', UploadVideoList.as_view(), name='st_video_list'),  # 文件列表
    # re_path(r'^_upload_image/?$', UploadImage.as_view(), ),

]



from rest_framework.views import APIView

from ..services.resource_file_service import ResourceFileService
from ..utils.custom_tool import request_params_wrapper
from ..utils.model_handle import util_response


class UploadFileList(APIView):
    # 文件列表
    @request_params_wrapper
    def get(self, *args, request_params, **kwargs):
        data, err = ResourceFileService.get_list(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->resource_oss_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/11/27 15:47
"""

# 获取授权策略：
# auth = oss2.Auth('LTAI5t5onsy3ZdzvgwJw3va8', 'DoSjbOVo4Hy6PiGwzEDojuB0paYK2D')
# bucket = oss2.Bucket(auth, 'http://oss-cn-beijing.aliyuncs.com', 'sunkaiyan')
# result = bucket.get_bucket_policy()
# policy_json = json.loads(result.policy)
# print("Get policy text: ", policy_json)

from aliyunsdkcore.client import AcsClient
from aliyunsdksts.request.v20150401.AssumeRoleRequest import AssumeRoleRequest

# 构建一个阿里云客户端，用于发起请求。
# 设置调用者（RAM用户或RAM角色）的AccessKey ID和AccessKey Secret。
client = AcsClient('LTAI5tJ3dWCHKsgvDMoNwp1f', 'RHgKlgTBOHq23B3cXmQu7JvV7ukjFR', 'cn-beijing')

# 构建请求。
request = AssumeRoleRequest()
request.set_accept_format('json')

# 设置参数。
request.set_RoleArn("acs:ram::1653068078015058:role/oss-service")
request.set_RoleSessionName("hhoss")

# 发起请求，并得到响应。
response = client.do_action_with_exception(request)
# python2:  print(response)
print(str(response, encoding='utf-8'))



# coding=utf-8
from django.core.paginator import Paginator, EmptyPage

from ..models import ResourceImage
from ..utils.custom_tool import force_transform_type, filter_fields_handler, format_params_handle


# 用于异常处理
def robust(actual_do):
    def add_robust(*args, **keyargs):
        try:
            return actual_do(*args, **keyargs)
        except Exception as e:
            print(str(e))

    return add_robust


class ResourceImageService:
    file_fields = [i.name for i in ResourceImage._meta.fields] + ["group_id"]

    def __init__(self):
        pass

    @staticmethod
    def get(image_id):
        image_set = ResourceImage.objects.filter(id=image_id).values(
            "id",
            "group_id",
            "user_id",
            "title",
            "filename",
            "url",
            "format",
            "size",
            "thumb",
            "md5",
            "snapshot",
            "counter",
        ).first()
        if not image_set:
            return None, '数据库找不到图片'
        return image_set, None

    @staticmethod
    def add(params):
        # print("> ResourceImageService create:", params)
        # # 检查MD5是否有历史上传记录，暂不使用，因为目录按月份分组时这些数据变得不稳定
        # md5 = params.get('md5', None)
        # if md5 is None:
        #     return False
        # instance = ResourceImage.objects.filter(md5=md5).first()
        # if instance:
        #     return instance, None

        # sieyoo注，数据写入最好通过显式的方式，这样可以随时修改和扩展复杂的需求，代价是必须和模型一致，另一个好处是开发者加深数据库结构的记忆
        new_params = {
            'group_id': params.get('group_id', None),
            'user_id': params.get('user_id', None),
            'title': params.get('title', None),
            'url': params.get('url', None),
            'filename': params.get('filename', None),
            'format': params.get('format', None),
            'size': params.get('size', None),
            'thumb': params.get('thumb', None),
            'md5': params.get('md5', None),
            'snapshot': params.get('snapshot', None),
            'counter': params.get('counter', 1),
        }

        try:
            image_set = ResourceImage(**new_params)
            image_set.save()
        except Exception as e:
            return None, f"""图片保存失败，请检查数据库。 {str(e)} in "{str(e.__traceback__.tb_frame.f_globals["__file__"])}" : Line {str(e.__traceback__.tb_lineno)}"""

        return image_set, None

    @staticmethod
    def get_list(params: dict = None):
        """图片分类查询"""
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        limit, is_pass = force_transform_type(variable=params.pop('limit', params.pop('size', 10)), var_type="int", default=10)
        page, is_pass = force_transform_type(variable=params.pop('page', 1), var_type="int", default=1)
        filter_fields = filter_fields_handler(
            input_field_expression=params.get("filter_fields"),
            default_field_list=[
                "group_id", "user_id", "title", "url", "filename", "format", "size", "thumb", "md5", "created_at", "updated_at"
            ],
            all_field_list=ResourceImageService.file_fields
        )

        # 参数过滤
        search_params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "group_id|int", "user_id|int", "title", "url", "filename", "format", "size", "md5",
                "created_start|date", "updated_start|date", "created_end|date", "updated_end|date",
                "counter",
            ],
            alias_dict={
                "title": "title__contains",
                "created_start": "created_at__gte",
                "updated_start": "updated_at__gte",
                "created_end": "created_at__lte",
                "updated_end": "updated_at__lte"
            }
        )

        # 构建ORM
        list_obj = ResourceImage.objects.extra(select={
            'created_at': 'DATE_FORMAT(created_at, "%%Y-%%m-%%d %%H:%%i:%%s")',
            'updated_at': 'DATE_FORMAT(updated_at, "%%Y-%%m-%%d %%H:%%i:%%s")'
        }).filter(**search_params)
        count = list_obj.count()
        list_obj = list_obj.values(*filter_fields)

        # 分页查询
        paginator = Paginator(list_obj, limit)
        try:
            res_set = paginator.page(page)
        except EmptyPage:
            return {'count': count, 'page': page, 'limit': limit, "list": []}, None

        page_list = []
        if res_set:
            page_list = list(res_set.object_list)

        return {'count': count, 'page': page, 'limit': limit, "list": page_list}, None
